#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <allegro.h>

#include "config.h"

char server[256];
uint16_t port;
char username[16];
uint16_t screen_w,
         screen_h;
uint8_t fullscreen;

const uint16_t buffer_w = 912/*800*/, buffer_h = /*600*/684;

void configDefault()
 {
  strncpy(server, "smart-idiot.no-ip.com", sizeof server);
  port = 2020;
  strncpy(username, "guest", sizeof username);
  screen_w = 800;
  screen_h = 600;
  fullscreen = 1;
 }

int configValid()
 {
  int c;
  
  for(c = 0; ; ++c)
   if(c == sizeof server)
    return 0;
   else if(!server[c])
    break;
   else if(!isprint(server[c]))
    return 0;
  
  for(c = 0; ; ++c)
   if(c == sizeof username)
    return 0;
   else if(!username[c])
    break;
   else if(!isprint(username[c]))
    return 0;
  
  if(port == 0)
   return 0;
  
  if(screen_w < 640)
   return 0;
  
  if(screen_h < 480)
   return 0;
  
  if(fullscreen != 0 && fullscreen != 1)
   return 0;
  
  return 1;
 }

int configLoad(const char *filename)
 {
  FILE *file = fopen(filename, "rb");
  
  if(!file)
   return 0;
  
  if(!fread(&server, sizeof server, 1, file) ||
     !fread(&port, sizeof port, 1, file) ||
     !fread(&username, sizeof username, 1, file) ||
     !fread(&screen_w, sizeof screen_w, 1, file) ||
     !fread(&screen_h, sizeof screen_h, 1, file) ||
     !fread(&screen_w, sizeof screen_w, 1, file) ||
     !fread(&fullscreen, sizeof fullscreen, 1, file))
   {
    fclose(file);
    return 0;
   }
  else
   {
    fclose(file);
    return 1;
   }
 }

int configSave(const char *filename)
 {
  FILE *file = fopen(filename, "wb");
  
  if(!file)
   return 0;
  
  if(!fwrite(&server, sizeof server, 1, file) ||
     !fwrite(&port, sizeof port, 1, file) ||
     !fwrite(&username, sizeof username, 1, file) ||
     !fwrite(&screen_w, sizeof screen_w, 1, file) ||
     !fwrite(&screen_h, sizeof screen_h, 1, file) ||
     !fwrite(&screen_w, sizeof screen_w, 1, file) ||
     !fwrite(&fullscreen, sizeof fullscreen, 1, file))
   {
    fclose(file);
    return 0;
   }
  else
   {
    fclose(file);
    return configValid();
   }
 }

static void askString(const char *question, char *buf, int size)
 {
  char *pos = buf + strlen(buf);
  
  clear_to_color(screen, makecol32(128, 0, 0));
  
  #ifdef OLD_ALLEGRO
  textout_centre(screen, font, "First time configure.", SCREEN_W/2, 0, makecol32(255, 255, 255));
  textout_centre(screen, font, "(Hit return to enter settings.)", SCREEN_W/2, 8, makecol32(255, 128, 0));
  textout_centre(screen, font, question, SCREEN_W/2, SCREEN_H/2-6, makecol32(255, 255, 255));
  #else
  textout_centre_ex(screen, font, "First time configure.", SCREEN_W/2, 0, makecol32(255, 255, 255), -1);
  textout_centre_ex(screen, font, "(Hit return to enter settings.)", SCREEN_W/2, 8, makecol32(255, 128, 0), -1);
  textout_centre_ex(screen, font, question, SCREEN_W/2, SCREEN_H/2-6, makecol32(255, 255, 255), -1);
  #endif
  
  clear_keybuf();
  
  do
   {
    rectfill(screen, 1, SCREEN_H/2+3, SCREEN_W-2, SCREEN_H/2+12, makecol32(0, 0, 0));
    
    #ifdef OLD_ALLEGRO
    textout_centre(screen, font, buf, SCREEN_W/2, SCREEN_H/2+4, makecol32(255, 255, 255));
    #else
    textout_centre_ex(screen, font, buf, SCREEN_W/2, SCREEN_H/2+4, makecol32(255, 255, 255), -1);
    #endif
    
    while(1)
     {
      poll_keyboard();
      
      if(keypressed())
       {
        int val = readkey();
        
        if((val >> 8) == KEY_BACKSPACE)
         {
          if(pos > buf)
           {
            --pos;
            *pos = 0;
            break;
           }
         }
        else if((val >> 8) == KEY_ENTER || (val >> 8) == KEY_ENTER_PAD)
         return;
        else if(isprint(val&0xff))
         {
          if(pos+1 != buf+size)
           {
            *pos = val&0xff;
            ++pos;
            *pos = 0;
            break;
           }
         }
       }
     }
   } while(1);
  
 }

static void askNumber(const char *question, uint16_t *number)
 {
  char buf[16];
  snprintf(buf, sizeof buf, "%d", *number);
  askString(question, buf, sizeof buf);
  *number = strtol(buf, 0, 10);
 }

static void askBool(const char *question, uint8_t *state)
 {
  char buf[16];
  strncpy(buf, *state ? "yes" : "no", sizeof buf);
  askString(question, buf, sizeof buf);
  *state = ((*buf == 'y' || *buf == 'Y') ? 1 : 0);
 }

int configCreate(const char *filename)
 {
  configDefault();
  
  do askString("Server name:", server, sizeof server); while(!configValid());
  do askNumber("Server port:", &port); while(!configValid());
  do askString("Username:", username, sizeof username); while(!configValid());
  do askNumber("Screen width:", &screen_w);  while(!configValid());
  screen_h = screen_w * 3 / 4;
  do askNumber("Screen height:", &screen_h); while(!configValid());
  do askBool("Fullscreen:", &fullscreen); while(!configValid());
  
  return configSave(filename);
 }
